#!/usr/bin/env python3
"""
   utils.configuration.app_configuration.py file for the c3buttons system.
   In this file the configuration will be setted from this order:
        config.yml -> environment vars
"""

###
### DEPENDENCIES
###
import sys

### LOCAL
try:
    from .default_values import DEFAULT_VALUES
except ImportError as e:
    print("Unable to import 'utils.configuration.default_vals.py' from 'utils.configuration.app_configuration.py' file")
    sys.exit(1)

try:
    from .app_logger import set_logger_from_default
except ImportError as e:
    print("Unable to import 'setup_logger' from 'utils.configuration.app_logger.py' file at the 'utils.configuration.app_configuration.py'")
    sys.exit(1)

try:
    from .from_file_environment import ConfigurationFromDotEnv
except ImportError as e:
    print("Unable to import 'ConfigurationFromFile' from 'utils.configuration.from_file_environment.py' file at the 'utils.configuration.app_configuration.py'")
    sys.exit(1)

try:
    from .from_file_yaml import ConfigurationFromYamlFile
except ImportError as e:
    print("Unable to import 'ConfigurationFromYamlFile' from 'utils.configuration.from_file_yaml.py' file at the 'utils.configuration.app_configuration.py'")
    sys.exit(1)


class AppConfiguration():
    """
        Class for setting the configuration of the app.
    """

    def __init__(self) -> None:
        """
            Function for initialiazing the AppConfiguration class.
            Sets the class variables.

            Returns
            -------
            None
        """
        self.logger = set_logger_from_default(name=__name__, values_default=DEFAULT_VALUES)

        self.logger.info("---> STARTED")

        self.__configuration_colors__()

        self.set_attr_from_parsed()

        if self.debug:
            self.__debug_attrs__()

    def check_missing_attrs(self, from_env: dict, from_yaml: dict) -> dict:

        self.logger.debug("check_missin_attrs")

        ret_dict = {}

        # If in from_env and not in from_yaml
        for name, value in from_env.dict_values.items():

            if name not in from_yaml.dict_values.keys():
                ret_dict[name] = value

            # if name in from_yaml.dict_values.keys():
            else:

                if name == "boxes":
                    yaml_value = from_yaml.dict_values.get("boxes")
                    if len(value) >= len(yaml_value):
                        boxes_dict = value
                        boxes_list = from_env.dict_values.get("boxes_used")
                        self.logger.info("---> Using boxes configuration from OS environments")

                    else:
                        boxes_dict = yaml_value
                        boxes_list = from_yaml.dict_values.get("boxes_used")
                        self.logger.info("---> Using boxes configuration from config.yml file")

                    # ret_dict[name] = new_value
                    ret_dict["boxes"] = boxes_dict
                    ret_dict["boxes_used"] = boxes_list

                elif name == "boxes_used":
                    pass

                else:
                    ret_dict[name] = value


            ### DEBUG
            # from pprint import pp
            # print(f"name: {name} | ret_dict: {ret_dict}")

        # If in from_yaml and not in from_env
        for name, value in from_yaml.dict_values.items():
            if name not in ret_dict.keys():
                ret_dict[name] = value

        # from pprint import pp
        # import pdb; pdb.set_trace()
        # pp(ret_dict)

        return ret_dict

    def attr_set_from_dict(self, dict_values: dict) -> None:

        self.logger.debug("attr_set_from_dict")

        for name, value in dict_values.items():
            setattr(self, name.lower(), value)

    def set_attr_from_parsed(self) -> None:

        self.logger.debug("set_attr_from_parsed")

        ### Get attrs from env
        from_env = ConfigurationFromDotEnv(default_values= DEFAULT_VALUES)

        ### If not using, set it from the os.env
        if not DEFAULT_VALUES.get("DEFAULT_USING_PARSED_YAML"):
            dict_values = from_env.dict_values

        else:
            ### Fetch params from YAML file
            from_yaml = ConfigurationFromYamlFile(default_values= DEFAULT_VALUES)
            dict_values = self.check_missing_attrs(from_env=from_env, from_yaml=from_yaml)

        ### Set the attributes as a dict aswell
        self.attr_set_from_dict(dict_values)

        self.dict_values = dict_values

    def __configuration_colors__(self) -> None:
        """
            Method for setting the terminal output with colors.

            Returns
            -------
            None
        """
        self.logger.debug("__configuration_colors__")

        self.color_gray = "\033[90m"
        self.color_red = "\033[91m"
        self.color_green = "\033[92m"
        self.color_orange = "\033[93m"
        self.color_blue = "\033[94m"
        self.color_pink = "\033[95m"
        self.color_yellow = "\033[93m"
        self.color_end = "\033[0m"

    def __debug_attrs__(self) -> None:
        """
            Function for printing the configuration parameters from the app.

            Returns
            -------
            None
        """
        self.logger.debug("__debug_attrs__")
        ### PRINT ARGS
        print("\n" + self.color_blue + "CONFIGURATION ATTRS:" + self.color_end)
        for name, value in self.dict_values.items():
            print(self.color_green + f"\t- {name}: " + self.color_end + f"'{value}'")

        print("\n")
