#!/usr/bin/env python3
"""
   utils.configuration.from_file.py file for the c3 buttons.
"""

###
### DEPENDENCIES
###
import sys
import os
import json

try:
    from dotenv import load_dotenv
except ImportError as e:
    print("Unable to import 'dotenv'")
    print("Please install it with 'pip(3) install dotenv --user'")
    sys.exit(1)

### LOCAL
try:
    from .from_file import ConfigurationFromFile
except ImportError as e:
    print("Unable to import 'ConfigurationFromFile' from 'utils.configuration.from_file.py' file at 'utils.configuration.from_file_environment.py' file" )
    sys.exit(1)


class ConfigurationFromDotEnv(ConfigurationFromFile):
    """ TODO
    """

    def __init__(self, default_values: dict) -> None:
        """
            TODO: Documentation
        """

        ### Init super
        super().__init__(name=__name__, default_values=default_values)

        self.path_config_file = self.default_values.get('DEFAULT_PATH_CONFIG_ENV')
        self.dict_values["boxes"] = {}
        self.get_values()

    def __get_env_bool__(self, name_config: str, name_config_default: str) -> bool:
        """
            Method to fetch the environment variable from a boolean type

            Returns
            -------
            bool:
                Return a boolean value from fetching the setted environment variable
        """
        self.logger.debug(f"__get_env_bool__ - {name_config}")
        try:
            fetched = os.getenv(name_config)

            if fetched:
                if fetched in ('YES', 'yes', 'true', 'True', 'TRUE'):
                    ret_bool = True
                elif fetched in ('NO', 'no', 'false', 'False', 'FALSE'):
                    ret_bool = False
                else:
                    print(f"\n\n Unable to fetch the parsed environment '{name_config}'. It is set but not 'true' or 'false' \n\n")
                    sys.exit(1)
            else:
                ret_bool = self.default_values.get(name_config_default)

        except Exception as e:
            print(f"\n\n Unable to fetch the parsed environment '{name_config}' \n| The exception is: '{e}' \n\n")
            sys.exit(1)

        return ret_bool

    def __get_env__(self, name_config: str, name_config_default: str) -> (str | int):
        """
            Method to fetch the environment variable from a string or int type.

            Returns
            -------
            str | int:
                Return a the value from fetching the setted environment variable.
        """
        self.logger.debug(f"__get_env__ - {name_config}")
        try:
            fetched = os.getenv(name_config)

            ### If not setted, get it from config
            if fetched is None:
                fetched = self.default_values.get(name_config_default)

        except Exception as e:
            print(f"\n\n Unable to fetch the parsed environment '{name_config}' \n| The exception is: '{e}' \n\n")
            sys.exit(1)

        return fetched

    def __get_env_dict__(self, name_config: str, name_config_default: str):
        """
            Method to fetch the environment variable from a string or int type.

            Returns
            -------
            str | int:
                Return a the value from fetching the setted environment variable.
        """
        self.logger.debug(f"__get_env_dict__ - {name_config}")

        fetched = self.__get_env__(
            name_config=name_config,
            name_config_default=name_config_default
        )

        try:
            ret_dict = json.loads(fetched)

        except Exception as e:
            print(f"\n\n Unable to parse the dictionary environment from: '{name_config}' \n| The exception is: '{e}' \n\n")
            sys.exit(1)

        self.dict_values["boxes"][name_config.lower()] = ret_dict

    def get_values(self) -> None:
        """
            Used for fetching the parsed values from the system.
            If the value is not parsed, it will use the default values settend at: utils.configuration.default_vals.py
        """
        self.logger.debug("get_values")

        if not load_dotenv(dotenv_path=self.path_config_file):
            print("\n\n  ERROR!: \ņ\n  Unable to load the '.env' file at the path: '{self.path_config_file}'. Change it at the '.configuration.default_vals.py' file")
            sys.exit()

        self.dict_values["description"] = self.__get_env__(
            name_config="DESCRIPTION",
            name_config_default="DEFAULT_DESCRIPTION"
        )
        ###
        ### DEBUG/VEBOSE
        ###
        self.dict_values["debug"] = self.__get_env_bool__(
            name_config="DEBUG",
            name_config_default="DEFAULT_DEBUG"
        )
        self.dict_values["verbose"] = self.__get_env_bool__(
            name_config="VERBOSE",
            name_config_default="DEFAULT_VERBOSE"
        )
        ###
        ### LOG
        ###
        self.dict_values["log_file_append_mode"] = self.__get_env_bool__(
            name_config="LOG_FILE_APPEND_MODE",
            name_config_default="DEFAULT_LOG_FILE_APPEND_MODE"
        )
        self.dict_values["log_to_file"] = self.__get_env_bool__(
            name_config="LOG_TO_FILE",
            name_config_default="DEFAULT_LOG_TO_FILE"
        )
        self.dict_values["log_file_path"] = self.__get_env__(
            name_config="LOG_FILE_PATH",
            name_config_default="DEFAULT_LOG_FILE_PATH"
        )
        self.dict_values["log_format"] = self.__get_env__(
            name_config="LOG_FORMAT",
            name_config_default="DEFAULT_LOG_FORMAT"
        )
        ###
        ### DB
        ###
        database_type = self.__get_env__(
            name_config="DATABASE_TYPE",
            name_config_default="DEFAULT_DATABASE_TYPE"
        )

        if database_type == "local_file":
            self.dict_values["database_table_name_button_box"] = self.__get_env__(
                name_config="DATABASE_TABLE_NAME_BUTTON_BOX",
                name_config_default="DEFAULT_DATABASE_TABLE_NAME_BUTTON_BOX"
            )
            self.dict_values["database_table_name_button_box_use"] = self.__get_env__(
                name_config="DATABASE_TABLE_NAME_BUTTON_BOX_USE",
                name_config_default="DEFAULT_DATABASE_TABLE_NAME_BUTTON_BOX_USE"
            )
            self.dict_values["database_local_db_file_path"] = self.__get_env__(
                name_config="DATABASE_LOCAL_DB_FILE_PATH",
                name_config_default="DEFAULT_DATABASE_LOCAL_DB_FILE_PATH"
            )

        elif database_type == "api":
            self.dict_values["database_api_url"] = self.__get_env__(
                name_config="DATABASE_API_URL",
                name_config_default="DEFAULT_DATABASE_API_URL"
            )
            self.dict_values["database_api_token"] = self.__get_env__(
                name_config="DATABASE_API_TOKEN",
                name_config_default="DEFAULT_DATABASE_API_TOKEN"
            )

        else:
            print(f"\n\n Unable to set the database_type from '{database_type}'\n Accepted values are: 'local_file' or 'api' \n\n")
            sys.exit(1)

        self.dict_values["database_type"] = database_type        

        ###
        ### BOXES CONFIG
        ###
        self.dict_values["box_button_bounce_time"] = self.__get_env__(
            name_config="BOX_BUTTON_BOUNCE_TIME",
            name_config_default="DEFAULT_BOX_BUTTON_BOUNCE_TIME"
        )
        self.dict_values["box_led_enabled"] = self.__get_env_bool__(
            name_config="BOX_LED_ENABLED",
            name_config_default="DEFAULT_BOX_LED_ENABLED"
        )
        self.dict_values["box_led_always_on"] = self.__get_env_bool__(
            name_config="BOX_LED_AWAYS_ON",
            name_config_default="DEFAULT_BOX_LED_AWAYS_ON"
        )        
        self.dict_values["box_led_blink_times"] = self.__get_env__(
            name_config="BOX_LED_BLINK_TIMES",
            name_config_default="DEFAULT_BOX_LED_BLINK_TIMES"
        )        
        ###
        ### BOXES USED
        ###
        # Set boxes_used as a list
        self.dict_values["boxes_used"] = self.__get_env__(
            name_config="BOXES_USED",
            name_config_default="DEFAULT_BOXES_USED"
        ).replace(' ','').replace("'", "").replace('"', '').split(',')


        # import pdb; pdb.set_trace()
        # self.dict_values["boxes_used"]
        # os.getenv("BOXES_USED")
        # load_dotenv(dotenv_path=self.path_config_file)
        # # "'BUTTON_BOX_1, BUTTON_BOX_2'"

        # Adds dict_values with 'boxes' infos from 'boxes_used' list
        for box_name in self.dict_values.get("boxes_used"):

            self.app_description = self.__get_env_dict__(
                name_config=box_name,
                name_config_default="DEFAULT_" + box_name
            )

        # Fix lower boxes names from boxes_used list
        for index, item in enumerate(self.dict_values.get("boxes_used")):
            self.dict_values.get("boxes_used")[index] = item.lower()
